/* ======================================================================= *
 *                                               .:;tjfLLffji,             *
 *                                          :,itLDKW#########Kj,           *
 *                                    .:ijLGEW########KEEKKW###Kf,         *
 *                              .,ifLGEK###########KDLfjjjffGE###Ei        *
 *                    .:,;tLDEW#######WKEDGLfLK##KLjjjjjjjjjjjE###Wt       *
 *                :;jLDEK#######WKEDGLffjjjjjD###DjjjjjjjjjjjjLW###L.      *
 *            .;jGK#########WDGLffjjjjjjjjjjjD###DjjjjjjjjjjjjLW###G:      *
 *           GW###WKDDDDEK#####KDLjjjjjjjjjjjfK##WGfjjjjjjjjjjK###Ei       *
 *          i###WDLfjjjjjfLKW#####EGLfjjjjjjjjLK##WEGfjjjjjfLD###Kt.       *
 *         ;E##KLjjjjjjjjjjjfGEW#####KEGfjjjjjjfW####WEDDEEK###WG;         *
 *         f##WLjjjjjjjjjjjjjjjfLDW#####WGfjjjjjGW############Gi.          *
 *        .f##KLjjjjjjjjjjjjjjjjjjjfLDEW####WW########KGLft;,.             *
 *         iK##EfjjjjjjjjjjjjjjjjjjjjjjfGK#######WGjt;:                    *
 *         .j###KGfjjjjjjjjjjjjjjjjjjjjjjfLDKW###WELt,                     *
 *           jE#####DLfjjjjjjjjjjjjjjjjjjjjjjjfLEW#####Gj,.                *
 *            ,jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEK####WDj;              *
 *               ,tLW####WKLfjjjjjjjjjjjjjjjjjjjjjjjfLDW####WL             *
 *                  ;jGK####KEGfjjjjjjjjjjjjjjjjjjjjjjjfGEW###i            *
 *             .,;itLDEK#########WGLfjjjjjjjjjjjjjjjjjjjjjfD##Wt           *
 *          ,fGEKW#######WWEEEW#####KDLfjjjjjjjjjjjjjjjjjjjfK##G:          *
 *       .iGK############ELjjjfGDK#####KEfjjjjjjjjjjjjjjjjjfE##G:          *
 *     .fW###EGLffjfLDK####LjjjjjjfLDKW####WEGLfjjjjjjjjjjLK##Ki           *
 *    .jW###EjjjjjjjjjfGW##KfjjjjjjjjjfGE######KGfjjjjjfLDW###t.           *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjjjjjjjjjLDKW####WEEEEEK####WG             *
 *    tW###DjjjjjjjjjjjjfK##KfjjjjjjjjjjjjjfGE############KGi.             *
 *    ;E###EfjjjjjjjjjjjGW##DjjjjfLGDEEKW########KEGLfj,.                  *
 *    .jW###EjjjjjjjjjfDW##WDGEEKK#######WKEDLfj;,.                        *
 *     .jK###EGffjjfLDK############WEDGLj;,:.                              *
 *       .;LK#############WDLft;,:                                         *
 *          ,fGEKKKKEGfjt;,.                                               *
 *             ,jjjji,                                                     *
 *                                                                         *
 *                      Shocks Servlet Framework                          *
 *        Copyright (c) 2003, NRFX Technologies LLC (www.nrfx.com)         *
 *                        All Rights Reserved.                             *
 *                                                                         *
 *  Licensed under the Academic Free License version 2.0; you may not use  *
 *  this file except in compliance with the License. You may obtain a      *
 *  copy of the License at http://www.opensource.org/licenses/afl-2.0.php  *
 *                                                                         *
 *  Software distributed under the License is distributed on an "AS IS"    *
 *  BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express    *
 *  or implied. See the License for the specific language governing        *
 *  permissions and limitations under the License.                         *
 * ======================================================================= */

package shocks.client;

import shocks.dawp.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.log4j.*;

/**
 * <p>Filters are a special type of <code>Action</code>, designed to add 
 * crosscutting functionality to the system.  The basic idea behind a Filter
 * is that it executes logic which <em>crosscuts</em> other actions in the
 * system.</p>
 * 
 * <p>More specifically, the <em>concerns</em> of Filter components crosscut
 * the entire application.  Internationalization (i18n), action-level security,
 * OR mappings, metrics and tracking, custom error handling, alerts--these
 * are things which potentially affect <em>every</em> action in the sytem.</p>
 * 
 * <p>And the system is trained to assemble sequences of filters which perform
 * these crosscutting concerns on actions, as specified in the workflow
 * descriptor at the time a module is loaded into the application.</p>  When
 * the concerns change, a new module can be assembled which alters the
 * filtration mechanisms to reflect the new process.</p>
 * 
 * <p>In all other regards, filters are just another type of 
 * <code>Action</code> object.  They have access to the same information that 
 * target actions have access to, but the information they pass back to the 
 * workflow components can actually interrupt the flow of control and prevent 
 * it from reaching a target action at all.  For example, a security filter can 
 * interrupt the workflow before it ever reaches the target action (which would
 * otherwise call the database). Or, a logging filter can track the number of 
 * times certain target actions are performed, allowing the system to gather 
 * metrics data. Another filter could negotiate <code>Locale</code> information
 * from the browser and retrieve information from resource bundles.</p>
 * 
 * <p>So, filters allow the introduction of system-level concerns which can be
 * applied in any order on the way into or out of a workflow sequence.</p>
 *
 */
public abstract class FilterSupport implements Action {
	private Logger log = Logger.getLogger("Filter");
	private String filterName;
	private String filterVersion;
	private ServletContext ctx;
	private String status;
	private Result result = null;
	
	/**
	 * Tells the enclosing <code>Filter</code> to proceed with the workflow
	 * sequence.
	 */
	public final String SUCCESS = "SUCCESS";
	
	/**
	 * Tells the enclosing <code>Filter</code> to terminate the workflow
	 * sequence and return an error to the <code>WorkflowProcessor</code>.
	 */
	public final String FAILURE = "FAILURE";
	
	/**
	 * Returns the name of the enclosing <code>Filter</code> object.
	 * @return String
	 */
	protected String getFilterName() {
		return this.filterName;
	}
	
	/**
	 * Returns the version of the enclosing <code>Filter</code> object.
	 * @return String
	 */
	protected String getFilterVersion() {
		return this.filterVersion;
	}
	
	/**
	 * Staffs the filter with metadata from its enclosing <code>Filter</code>
	 * object, as well as data concerning its status in the workflow sequence.
	 * 
	 * @param metadata
	 */
	public final void setFilterMetadata(DataSource metadata, String status) {
		this.filterName = (String)metadata.getAttribute("name");
		this.filterVersion = (String)metadata.getAttribute("version");
		this.status = status;
	}
	
	/**
	 * Provides access to the <code>ServletContext</code>.
	 * @return ServletContext
	 */
	protected ServletContext getServletContext() {
		return this.ctx;
	}
	
	/**
	 * <p><strong>This method is <code>final</code> and cannot be overridden 
	 * by the user</strong>. It is specifically designed for use by the 
	 * container.  The method the user should concern themselves with is below.
	 * </p>
	 * 
	 * <p>This method is called by the enclosing <code>Filter</code> object.  
	 * Its purpose is to staff the action with a request & response pair, and 
	 * a reference to the current servlet context.  It also allows the 
	 * designers of the framework to change the <code>Action</code> interface 
	 * at a later time without forcing the users to alter the actions they've
	 * already built.</p>
	 * 
	 * @return DataSource
	 */
	public synchronized final DataSource execute(		
		HttpServletRequest request, 
		HttpServletResponse response,
		ServletContext ctx) {
		
		this.ctx = ctx;
		Result result = new Result(execute(request, response));
		this.status = null;
		// this.result = null;
		return result;
	}
	
	public final void setResult(Result result) {
		this.result = result;
	}
	
	public final Result getResult() {
		return result;
	}
	
	/**
	 * <p>Developers are required to override this method when writing 
	 * crosscutting actions.  They may perform operations on the request and
	 * response objects, make use of the servlet context, get relevant data
	 * about the workflow sequence currently in progress, and pass data back
	 * and forth between filter components by using the FilterResult object.
	 * </p>
	 * 
	 * <p>This method must return a command string to its enclosing 
	 * <code>Filter</code> object.  There are two standard results which are 
	 * allowed by the framework: SUCCESS and FAILURE. An example of their use 
	 * follows:</p>
	 * 
	 * <p><code>return SUCCESS;</code></p>
	 * 
	 * <p>This tells the workflow processor that execution of the FilterSupport
	 * class has been successful and that it should proceed to the next step in
	 * the workflow sequence.  If any string value other than SUCCESS is returned
	 * to the container, it will assume that the workflow sequence has been
	 * terminated and return control to the <code>WorkflowController</code>.</p>
	 * 
	 * @param request
	 * @param response
	 * @return String
	 */
	protected abstract String execute(
		HttpServletRequest request, 
		HttpServletResponse response);
	
	protected boolean isIncoming() {
		return status.equals("incoming");
	}
	
	protected boolean isOutgoing() {
		return status.equals("outgoing");
	}
}